package api_we

import (
	"dc_golang_server_1/api"
	"dc_golang_server_1/data_db_cache/cache"
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/logger"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
)

type creatCardUserRes struct {
	ID      uint32   `json:"id"`                            // 新创建的学生ID
	Service [6]uint8 `json:"service" example:"1,2,2,2,2,2"` // 对应6个数字对应公话、定位、饮水、卡消费、洗浴、洗衣机图标(0不显示,1正常,2显示但提示校区暂未开通该服务)
}

// CreateCardUser 创建一个学生
// @Tags 家长微信
// @Summary  创建一个学生 (同一微信号并发创建学生,WechatCustomerMap缓存弱安全) [complete]
// @Description 传新建学生信息至后台,后台返回学生id,以及对应校区的service
// @Accept  json
// @Produce json
// @Param data body dbdao.InsertTableUserCardUser true "学生信息"
// @Success 0 {object} creatCardUserRes
// @Router /we/carduser [POST]
// @Security ApiKeyAuth
func CreateCardUser(c *gin.Context) {
	var reqData dbdao.InsertTableUserCardUser
	err := c.ShouldBindJSON(&reqData)
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    api.ErrorReqParaFormat,
			"message": api.CodeMsg[api.ErrorReqParaFormat],
		})
		return
	}
	v, ok := c.Get("userID")
	weUserID := v.(uint32)
	if ok != true || weUserID == 0 {
		c.JSON(http.StatusOK, gin.H{
			"code": api.ErrorSystemErr,
			"data": api.CodeMsg[api.ErrorSystemErr],
		})
		logger.Log.Error("strconv.Atoi(c.Param(\"userID\")",
			zap.String("c.Param(\"userID\")", c.Param("userID")),
			zap.Error(err))
		return
	}

	newID := dbdao.InsertUserCardUserAndUserWeCardUser(reqData, weUserID) //插入数据库
	if newID == 0 {
		c.JSON(http.StatusOK, gin.H{
			"code": api.ErrorSystemErr,
			"data": api.CodeMsg[api.ErrorSystemErr],
		})
		return
	}
	//更新微信用户缓存，添加该学生
	if cache.InsertNewStudentIDToWechatCustomerMap(weUserID, newID, true) == false {
		c.JSON(http.StatusOK, gin.H{
			"code": api.ErrorSystemErr,
			"data": api.CodeMsg[api.ErrorSystemErr],
		})
		return
	}
	//添加学生缓存
	cache.CardUserMap.Store(newID, cache.CardUserType{
		AreaID:             reqData.AreaID,
		Name:               reqData.Name,
		Sex:                reqData.Sex,
		Grade:              reqData.Grade,
		Class:              reqData.Class,
		StuNum:             reqData.StuNum,
		MasterWechatUserID: weUserID,
	})

	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": creatCardUserRes{
			newID,
			[6]uint8{1, 2, 2, 2, 2, 2},
		},
	})
}
