package api_we

import (
	"dc_golang_server_1/api"
	"dc_golang_server_1/data_db_cache/cache"
	"dc_golang_server_1/data_db_cache/dbdao"
	"dc_golang_server_1/data_db_cache/model"
	tencent2 "dc_golang_server_1/tencent"
	"fmt"
	"github.com/gin-gonic/gin"
	"net/http"
	"time"
)

type WeSlaveLoginResUserInfoStruct struct {
	ID      uint32   `json:"id"` // 学生ID
	Name    string   `json:"name"`
	Sex     uint8    `json:"sex"`                           // 0无意义 1女 2男
	Area    string   `json:"area" example:"温江实验中学"`         // 区域名字
	Grade   int8     `json:"grade"`                         // 0无意义 "1~9对应一年级~九年级，10~12对应高一~高三"
	Class   int8     `json:"class"`                         // 0无意义
	StuNum  string   `json:"stuNum"`                        // ""无意义
	Service [6]uint8 `json:"service" example:"1,2,2,0,0,0"` // 对应6个数字对应公话、定位、饮水、卡消费、洗浴、洗衣机图标(0不显示,1正常,2显示但提示校区暂未开通该服务)
	// TopUp bool `json:"topUp"` // 是否已充值(已充值则不能删除，未充值可以删除)
}
type WeMasterLoginResUserInfoStruct struct {
	WeSlaveLoginResUserInfoStruct
	TopUp bool `json:"topUp"` // 是否已充值(已充值则不能删除，未充值可以删除)
}

type weLoginRes struct {
	Master  []WeMasterLoginResUserInfoStruct `json:"master"`  // 自己创建的学生列表(主家长)
	Slave   []WeSlaveLoginResUserInfoStruct  `json:"slave"`   // 添加的学生列表(副家长)
	WePhone bool                             `json:"wePhone"` // 是否已获取到微信电话
	WeInfo  bool                             `json:"weInfo"`  // 是否已获取到微信用户信息
	Token   string                           `json:"token"`
}

type weLoginReq struct {
	Code  string `json:"code" example:"codeTest1"` //必填,登录要用的微信code
	Token string `json:"token,omitempty"`          //选填,如果有旧的token,也带过来
}

// WeLogin 微信登录
// @Tags 家长微信
// @Summary  登录 [complete]
// @Description 微信code传至后台,后台返回token,token有效时长:48小时
// @Param data body weLoginReq true "登录信息"
// @Product json
// @Success 0 {object} weLoginRes
// @Router /we/login/{code} [POST]
func WeLogin(c *gin.Context) {
	var reqData weLoginReq
	err := c.ShouldBindJSON(&reqData)
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    api.ErrorReqParaFormat,
			"message": api.CodeMsg[api.ErrorReqParaFormat],
		})
		return
	}

	var userID uint32
	nowTime := time.Now().UnixMicro()

	var tokenInfo api.TokenResult
	if len(reqData.Token) > 0 {
		tokenInfo = api.DecryptToken(reqData.Token)
		if tokenInfo.UserType == 1 {
			api.CheckTokenExit(&tokenInfo)
			if tokenInfo.CheckResult == api.Success {
				userID = tokenInfo.UserID
			}
		}
	}
	var userInfo model.CacheUWeCustomerStruct
	var haveInfo bool
	if userID != 0 {
		userInfo, haveInfo = cache.GetWechatCustomerInfoAndCanUpdateTokenTime(userID, nowTime)
		if haveInfo {
			dbdao.UpdateUWeCustomerLoginTime(userID)
		}
	}

	if haveInfo == false { // 微信服务器登录
		unionID, err := tencent2.TencentCode2Session(reqData.Code) //也不去校验reqData.Code长度了，对半是对的
		if err != nil {
			c.JSON(http.StatusOK, gin.H{
				"code":    api.ErrorTencentErr,
				"message": api.CodeMsg[api.ErrorTencentErr] + err.Error(),
			})
			return
		}
		userID = cache.GetWechatCustomerUserIDByUnionID(unionID)
		if userID == 0 { // 原来没有，只有注册
			userInfo.TokenCreatTime = nowTime
			userID = cache.InsertWechatCustomerMap(unionID, &userInfo)
			if userID == 0 {
				c.JSON(http.StatusOK, gin.H{
					"code":    api.ErrorSystemErr,
					"message": api.CodeMsg[api.ErrorSystemErr],
				})
				return
			}
		} else { // 已有用户
			userInfo, haveInfo = cache.GetWechatCustomerInfoAndCanUpdateTokenTime(userID, nowTime)
			if haveInfo {
				dbdao.UpdateUWeCustomerLoginTime(userID)
			} else {
				cache.WechatCustomerMap.Delete(userID)
				cache.WechatCustomerUnionIDToIDMAP.Delete(unionID)
				fmt.Println(time.Now(), "严重错误:微信用户ID", userID, "在WechatCustomerUnionIDToIDMAP有,而其他地方没有,已删除")
				c.JSON(http.StatusOK, gin.H{
					"code":    api.ErrorSystemErr,
					"message": api.CodeMsg[api.ErrorSystemErr],
				})
				return
			}
		}
	}

	var res weLoginRes
	res.Token = api.CreateToken(nowTime, userID, 1, 48*24*3600)
	if len(userInfo.WePhone) > 0 {
		res.WePhone = true
	}
	if len(userInfo.Nickname) > 0 { // todo 这里考虑长时间之后重新获取一盘昵称头像
		res.WeInfo = true
	}

	for _, cardUserID := range userInfo.MasterCardUserID {
		if cardUserInfo, ok := cache.GetCardUserInfo(cardUserID); ok {
			var areaInfo model.CacheBAreaStruct
			var upInfo WeMasterLoginResUserInfoStruct
			upInfo.ID = cardUserID
			upInfo.Name = cardUserInfo.Name
			upInfo.Sex = cardUserInfo.Sex
			upInfo.Grade = cardUserInfo.Grade
			upInfo.Class = cardUserInfo.Class
			upInfo.StuNum = cardUserInfo.StuNum
			upInfo.TopUp = true
			upInfo.Service = [6]uint8{1, 2, 2, 2, 2, 2}
			fmt.Println("upInfo.Service", upInfo.Service)
			if cache.GetAreaMapInfo(cardUserInfo.AreaID, &areaInfo) {
				if areaInfo.Status == 1 || (areaInfo.Status == 2 && userInfo.TestRole) {
					upInfo.Area = areaInfo.Name
				}
			}
			res.Master = append(res.Master, upInfo)
		}
	}

	for _, cardUserID := range userInfo.SlaveCardUserID {
		if cardUserInfo, ok := cache.GetCardUserInfo(cardUserID); ok {
			var areaInfo model.CacheBAreaStruct
			var upInfo WeSlaveLoginResUserInfoStruct
			upInfo.ID = cardUserID
			upInfo.Name = cardUserInfo.Name
			upInfo.Sex = cardUserInfo.Sex
			upInfo.Grade = cardUserInfo.Grade
			upInfo.Class = cardUserInfo.Class
			upInfo.StuNum = cardUserInfo.StuNum
			//upInfo.TopUp = true
			upInfo.Service = [6]uint8{1, 2, 2, 2, 2, 2}
			fmt.Println("upInfo.Service", upInfo.Service)
			if cache.GetAreaMapInfo(cardUserInfo.AreaID, &areaInfo) {
				if areaInfo.Status == 1 || (areaInfo.Status == 2 && userInfo.TestRole) {
					upInfo.Area = areaInfo.Name
				}
			}
			res.Slave = append(res.Slave, upInfo)
		}
	}

	c.JSON(http.StatusOK, gin.H{
		"code": api.Success,
		"data": res,
	})
}
